﻿Imports System.Windows.Forms.DataVisualization.Charting

Public Class Form1

    Private northwind As NorthwindEntities

    Private Sub PopulateDataSources()
        'Populates the CustomersBindingSource
        'with Customers from the model using
        'eager loading
        Me.CustomerBindingSource.DataSource = northwind.
                                              Customers.Include("Orders")

        'Populates the OrderBindingSource
        'establishing a relationships with Customers
        Me.OrdersBindingSource.DataSource = CustomerBindingSource
        Me.OrdersBindingSource.DataMember = "Orders"

        'To be set explicitly in EF
        Me.OrdersBindingSource.AllowNew = True
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object,
                           ByVal e As System.EventArgs) Handles MyBase.Load
        PopulateDataSources()
    End Sub

    Private Sub CustomerBindingNavigatorSaveItem_Click(ByVal sender As System.Object,
                                                       ByVal e As System.EventArgs) _
                                                       Handles CustomerBindingNavigatorSaveItem.Click

        Try
            Me.Validate()
            Me.OrdersBindingSource.EndEdit()
            northwind.SaveChanges()
        Catch ex As OptimisticConcurrencyException
            northwind.Refresh(Objects.RefreshMode.ClientWins,
                              northwind.Orders)
            northwind.SaveChanges()
        Catch ex As UpdateException
            Me.OrdersBindingSource.CancelEdit()
            MessageBox.Show(ex.Message)
        Catch ex As Exception
            Me.OrdersBindingSource.CancelEdit()
            MessageBox.Show(ex.ToString)
        End Try
    End Sub

    Private Sub CustomersBindingSource_CurrentChanged(ByVal sender As Object,
                                                   ByVal e As System.EventArgs) _
                                                   Handles CustomerBindingSource.
                                                   CurrentChanged

        'Get the instance of the current customer
        Dim currentItem As Customer = CType(Me.CustomerBindingSource.Current, 
                                            Customer)

        Dim source = Query(currentItem.CustomerID)
        Try
            'Required to refresh the chart
            Chart1.Series.Clear()
            'Groups by Year, shows the year on the X axis,
            'shows the total price on the Y axis, formats
            'the tooltip as currency
            Chart1.DataBindCrossTable(source, "Year",
                                              "Year",
                                              "TotalAmount",
                                              "Tooltip=TotalAmount{C2}")

        Catch ex As Exception
        End Try
    End Sub

    Function Query(ByVal CustomerID As String) As  _
             IEnumerable(Of CustomerData)

        Dim result = From cust In northwind.Customers.Include("Orders")
                     Where cust.CustomerID = CustomerID
                     Join ordr In northwind.Orders.Include("Order_Details") _
                     On ordr.CustomerID Equals cust.CustomerID
                     Let totalAmount =
                     Aggregate ord_det In ordr.Order_Details
                     Into Sum(ord_det.UnitPrice)
                     Select New CustomerData With {.TotalAmount = totalAmount,
                                                   .Year =
                                                   ordr.OrderDate.Value.Year}

        Return result.AsEnumerable
    End Function


    Private Sub ExportChartToolStripButton_Click(ByVal sender As System.Object,
                                                 ByVal e As System.EventArgs) _
                                                 Handles ExportChartToolStripButton.Click
        With SaveFileDialog1
            .Filter = "Jpeg files|*.jpg|Png files|*.png|" & _
                      "Bmp files|*.bmp|Tiff files|*.tiff|Gif files|*.gif"
            .ShowDialog()
            If String.IsNullOrEmpty(.FileName) = False Then
                Me.Chart1.SaveImage(.FileName,
                CType(.FilterIndex - 1, ChartImageFormat))
            End If
        End With
    End Sub

    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Me.northwind = New NorthwindEntities
    End Sub


    Private Sub OrdersBindingSource_DataError(ByVal sender As Object,
                                              ByVal e As System.Windows.Forms.
                                              BindingManagerDataErrorEventArgs) _
                                              Handles OrdersBindingSource.DataError
        MessageBox.Show(e.Exception.Message, "", MessageBoxButtons.OK,
                        MessageBoxIcon.Error)
    End Sub


End Class


