﻿Imports System.Xml.Linq
Imports System.IO, System.Text
Imports System.IO.IsolatedStorage

Partial Public Class MainPage
    Inherits UserControl

    'Declaring a books collection
    Private MyBooks As BooksCollection

    'Required for understanding if the DataGrid
    'is in edit mode
    Private isEditing As Boolean = False

    Public Sub New()

        InitializeComponent()
    End Sub

    'Used to generate data if the data file is not
    'found
    Private Function CreateData() As BooksCollection
        Dim b As New Book With {.ISBN = "0000000"}

        Dim bc As New BooksCollection
        bc.Add(b)
        Return bc
    End Function

    'Attempts to read the data file from the isolated storage
    'If found, with a LINQ to Xml query a new books collection
    'is returned. If not found, a new empty collection is 
    'generated and returned
    Private Function GetBooks() As BooksCollection
        Try
            Dim doc As XDocument

            Using store As IsolatedStorageFile = IsolatedStorageFile.
                                                 GetUserStoreForApplication
                Dim st As IsolatedStorageFileStream = _
                          store.OpenFile("Books.xml", FileMode.Open)
                doc = XDocument.Load(st)
                st.Close()
            End Using

            Dim query = From pbook In doc...<Book>
                      Select New Book With {.Author = pbook.@Author,
                                            .Title = pbook.@Title,
                                            .DatePublished = Date.
                                            Parse(pbook.@DatePublished),
                                            .ISBN = pbook.@ISBN
                      }

            Return New BooksCollection(query)

        Catch ex As Exception
            Return CreateData()
        End Try
    End Function


    Private Sub MainPage_Loaded(ByVal sender As Object,
                                ByVal e As System.Windows.
                                RoutedEventArgs) Handles Me.Loaded
        'Populates data
        Me.MyBooks = GetBooks()
        'Sets data-binding
        Me.DataContext = Me.MyBooks
    End Sub

    'Saves data to the isolated storage. The Xml data is generated
    'with LINQ to Xml embedded-expressions
    Private Sub SaveButton_Click(ByVal sender As System.Object,
                                 ByVal e As System.Windows.
                                 RoutedEventArgs) Handles SaveButton.Click
        Dim data = <?xml version="1.0" encoding="utf-8"?>
                   <Books>
                       <%= From b In MyBooks
                           Select <Book Author=<%= b.Author %>
                                      Title=<%= b.Title %>
                                      ISBN=<%= b.ISBN %>
                                      DatePublished=<%= b.DatePublished.
                                                        ToString %>/>
                       %>
                   </Books>

        Using store As IsolatedStorageFile = IsolatedStorageFile.
                                             GetUserStoreForApplication
            Dim st As IsolatedStorageFileStream = _
                      store.OpenFile("Books.xml", FileMode.Create)
            data.Save(st)
            st.Close()
        End Using
    End Sub


    Private Sub BooksGrid_BeginningEdit(ByVal sender As Object,
                                        ByVal e As System.Windows.Controls.
                                        DataGridBeginningEditEventArgs) Handles BooksGrid.
                                        BeginningEdit
        isEditing = True
    End Sub

    'Allows DataGrid editing
    Private Sub BooksGrid_KeyDown(ByVal sender As Object,
                                  ByVal e As System.Windows.Input.
                                  KeyEventArgs) Handles BooksGrid.KeyDown
        If isEditing = False Then
            'If the user press Delete, removes the selected item
            If e.Key = Key.Delete Then

                If Me.BooksGrid.SelectedItem IsNot Nothing Then
                    Me.MyBooks.Remove(CType(Me.BooksGrid.SelectedItem, Book))
                End If
                'If the user press Insert, adds a new empty item to the books collection
            ElseIf e.Key = Key.Insert Then
                Dim b As New Book With {.DatePublished = Today}
                Dim index As Integer = MyBooks.IndexOf(CType(Me.BooksGrid.
                                                             SelectedItem, Book))

                MyBooks.Insert(index + 1, b)
                BooksGrid.SelectedIndex = index
                BooksGrid.BeginEdit()
            End If
        End If
    End Sub


    Private Sub BooksGrid_RowEditEnding(ByVal sender As Object,
                                        ByVal e As System.Windows.Controls.
                                        DataGridRowEditEndingEventArgs) _
                                        Handles BooksGrid.RowEditEnding
        isEditing = False
    End Sub
End Class