﻿Imports System.Threading.Tasks
Imports System.Threading

Module Tasks

    Sub Main()
        BarrierDemo()
        Console.ReadLine()
        Dim sw As New Stopwatch
        sw.Start()
        'This comment will be replaced by
        'the method executing the loop
        ParallelForEachTest()
        sw.Stop()
        Console.WriteLine("Elapsed: {0}", sw.Elapsed)
        Console.ReadLine()
    End Sub

    Sub CreateTasks()

        Dim simpleTask As New Task(Sub()
                                       'Do your work here...
                                   End Sub)

        simpleTask.Start()

        simpleTask.Wait(1000)
        If simpleTask.IsCompleted Then
            'completed
        Else
            'timeout
        End If

        Dim taskA = Task.Factory.StartNew(Sub() Console.WriteLine("Task A"))
        Dim taskB = Task.Factory.StartNew(Sub() Console.WriteLine("Task B"))
        Dim taskC = Task.Factory.StartNew(Sub() Console.WriteLine("Task C"))
    End Sub

#Region "Cancellation"
    Sub CancellationDemo()
        Dim tokenSource As New CancellationTokenSource()
        Dim token As CancellationToken = tokenSource.Token

        Dim aTask = Task.Factory.StartNew(Sub() DoSomething(token), token)

        Thread.Sleep(1000)

        tokenSource.Cancel()

        Try
            aTask.Wait()
        Catch ex As AggregateException
            'Handle concurrent exceptions here...
        Catch ex As Exception

        End Try

        Console.WriteLine(aTask.status)
    End Sub

    Sub DoSomething(ByVal cancelToken As CancellationToken)

        'Check if cancellation was requested before
        'the task starts
        If cancelToken.IsCancellationRequested = True Then
            cancelToken.ThrowIfCancellationRequested()
        End If

        For i As Integer = 0 To 1000

            'Simulates some work
            Thread.SpinWait(10000)
            If cancelToken.IsCancellationRequested Then

                'Cancellation was requested
                cancelToken.ThrowIfCancellationRequested()
            End If
        Next

    End Sub

#End Region

#Region "Exception handling"

    Sub ExceptionDemo()

        Dim aTask = Task.Factory.StartNew(Sub() Console.WriteLine("A demo task"))

        Try
            aTask.Wait()
        Catch ex As AggregateException
            For Each fault In ex.Flatten.InnerExceptions
                If TypeOf (fault) Is InvalidOperationException Then
                    'Handle the exception here..
                ElseIf TypeOf (fault) Is NullReferenceException Then
                    'Handle the exception here..
                End If
            Next
        Catch ex As Exception

        End Try

    End Sub

#End Region

#Region "Tasks that return a value"

    Sub CreateTasksWithValue()

        Dim taskWithResult = Task(Of Double).
            Factory.StartNew(Function()
                                 Dim radians As Double _
                                     = 120 * Math.PI / 180
                                 Dim tan As Double = _
                                     Math.Tan(radians)
                                 Return tan
                             End Function)

        Console.WriteLine(taskWithResult.Result)
    End Sub

    Private Function CalcTan(ByVal angle As Double) As Double

        Dim tangentResult As Double

        Dim t = Task(Of Double).Factory.
            StartNew(Function()
                         Dim radians As Double = angle * Math.PI / 180
                         tangentResult = Math.Tan(radians)
                         Return tangentResult
                     End Function)
        Return t.Result
    End Function

#End Region

#Region "Task scheduler interaction"
    Private Sub TaskSchedulerInteractionDemo()

        Console.WriteLine("The maximum concurrency level is {0}",
                          TaskScheduler.Current.MaximumConcurrencyLevel)

    End Sub
#End Region


#Region "Barrier"

    Sub BarrierDemo()
        ' Create a barrier with three participants
        ' The Sub lambda provides an action that will be taken
        ' at the end of the phase
        Dim myBarrier As New Barrier(3,
                                   Sub(b)
                                       Console.
                                       WriteLine("Barrier has been " & _
                                                 "reached (phase number: {0})",
                                                         b.CurrentPhaseNumber)
                                   End Sub)

        ' This is the sample work made by all participant tasks
        Dim myAction As Action =
            Sub()
                For i = 1 To 3
                    Dim threadId As Integer = Thread.CurrentThread.ManagedThreadId
                    Console.WriteLine("Thread {0} before wait.", threadId)

                    'Waits for other tasks to arrive at this same point:
                    myBarrier.SignalAndWait()
                    Console.WriteLine("Thread {0} after wait.", threadId)
                Next
            End Sub

        ' Starts three tasks, representing the three participants
        Parallel.Invoke(myAction, myAction, myAction)

        ' Once done, disposes the Barrier.
        myBarrier.Dispose()
    End Sub

#End Region
End Module