﻿Imports Microsoft.Office.Interop.Word
Imports System.Windows.Forms

Public Class SummaryTaskPane
    Private Function GetRevisions() As XElement

        'Queries for available revisions
        'The ActiveDocument properties represents the active
        'document in Word and the Revisions property is a
        'collection of Word revisions
        Dim revisions = <Revisions>
                            <%= From rev As Revision In Globals.
                                ThisAddIn.Application.ActiveDocument.Revisions.
                                OfType(Of Revision)() _
                                Select _
                                <Revision Text=<%= rev.Range.Text %>
                                    Author=<%= rev.Author %>
                                    Timestamp=<%= rev.Date.ToString %>/> %>

                        </Revisions>

        Return revisions
    End Function

    Private Function GetComments() As XElement

        'Same as the previous method, but with regard
        'to comments
        Dim comments = <Comments>
                           <%= From comm As Word.Comment In Globals.ThisAddIn.
                               Application.ActiveDocument.Comments.
                               OfType(Of Comment)()
                               Select <Comment
                                          Text=<%= comm.Range.Text %>
                                          Author=<%= comm.Author %>
                                          Date=<%= comm.Date.ToString %>
                                      />
                           %>
                       </Comments>
        Return comments
    End Function

    'Creates a new Xml document adding the content
    'of the XElement objects returned by the
    'above methods
    Private Sub CreateSummary(ByVal fileName As String, ByVal comments As Boolean,
                              ByVal revisions As Boolean)

        If comments = False And revisions = False Then
            Throw New ArgumentException
        End If

        Dim doc As New XDocument
        doc.Declaration = New XDeclaration("1.0", "utf-8", "yes")

        Dim rootElement As New XElement("Summary")


        If comments Then rootElement.Add(GetComments)
        If revisions Then rootElement.Add(GetRevisions)

        doc.Add(rootElement)
        Try
            doc.Save(fileName)
            doc = Nothing
        Catch ex As Exception
            Throw
        End Try
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Dim saveFile As New SaveFileDialog
        With saveFile
            .Title = "Select the target"
            .Filter = "Xml files|*.xml|All files|*.*"

            If .ShowDialog = DialogResult.OK Then
                Try
                    Me.CreateSummary(.FileName, CommentsCheckBox.Checked,
                                     RevisionsCheckBox.Checked)

                Catch ex As Exception
                    MessageBox.Show(ex.Message)
                End Try
            End If
        End With
    End Sub
End Class
